/*******************************************************************************
BoundingDiameters
@author Frank Takes - ftakes@liacs.nl
@website http://www.liacs.nl/~ftakes/diameter
**************** consult the README.txt file before using *********************/

Graph::Graph() {
	cerr << "------------------------------------------------------------------" 
	 	 << endl << endl;
	N = 0;
	M = 0;
	largestWCC = 0;
	sorted = false;
	isUndir = false;
	memset(INDEG, 0, sizeof(INDEG));
	memset(OUTDEG, 0, sizeof(OUTDEG));	
	memset(COUNTINDEG, 0, sizeof(COUNTINDEG));
	memset(COUNTOUTDEG, 0, sizeof(COUNTOUTDEG));	
	memset(WCC, 0, sizeof(COUNTINDEG));
	memset(WCCnodes, 0, sizeof(COUNTOUTDEG));			
	for(int i=0; i<MAXN+2; i++) {
		E[i].clear();
	}
	cerr << "Graph initialized." << endl << endl;
} // Graph constructor

void Graph::setUndirEdge(int a, int b) {
	setEdge(a, b);
	setEdge(b, a);	
} // setUndirEdge

void Graph::setEdge(int a, int b) {
	if(a == b) {
		cerr << a << " EQUALS " << b << endl;
		return;
	}

	if(a == 0)
		a = MAXN + 1;
	if(b == 0)
		b = MAXN + 1;		
		
	E[a].push_back(b);

	if(a > N)
		N = a;
	if(b > N)
		N = b;
	COUNTINDEG[INDEG[a]]--;
	COUNTOUTDEG[OUTDEG[b]]--;	
	OUTDEG[a]++;
	INDEG[b]++;		
	COUNTINDEG[INDEG[a]]++;
	COUNTOUTDEG[OUTDEG[b]]++;	
	M++;
	sorted = false;
} // setEdge

bool Graph::edge(int a, int b) { 
	if(!sorted)
		sortEdgeList();
	int first = 0, last = E[a].size() - 1;
    while (first <= last) {
		int mid = (first + last) / 2;		
		if (b > E[a][mid]) 
			first = mid + 1;             	
		else if (b < E[a][mid]) 
			last = mid - 1; 				
		else
			return true;     				
	}
	return false;
} // edge

void Graph::sortEdgeList() {
	cerr << "Sorting Edge List..." << endl;
	for(int i=1; i<=N; i++) 
		sort(E[i].begin(), E[i].end());	 
	sorted = true;
	cerr << "Edge List Sorting complete." << endl << endl;
	calcWCC();
}

void Graph::outputStat(string s, int value) {
	cerr << setw(40) << s << setw(12) << value << endl;
} // outputStat

void Graph::print() {
	realN = 0;
	int realNout=0, realNin = 0, realNeq=0;
	for(int i=1; i<=N; i++) {
		if(OUTDEG[i] > 0)
			realNout++;
		if(INDEG[i] > 0)
			realNin++;			
		if(INDEG[i] > 0 || OUTDEG[i] > 0) {
			if(INDEG[i] == OUTDEG[i])
				realNeq++;		
			realN++;
		} // if
		
	} // for

	cerr << "------------------------------------------------------------------" 
	     << endl;
	outputStat("Number of Nodes", realN);	
	outputStat("Number of Nodes with OUTDEG > 0", realNout);		
	outputStat("Number of Nodes with INDEG > 0", realNin);		
	outputStat("Number of Nodes with INDEG = OUTDEG", realNeq);
	outputStat("Number of Nodes with INDEG != OUTDEG", realN-realNeq);
	cerr << endl;
	outputStat("Number of Edges = M", M);
	if(realN > 0) 	outputStat("Average in=out=degree", M/realN);
	cerr << endl;
	outputStat("Largest Node Number = N", N);
	cerr << endl;	
	if(sorted) {
		// output some stats
		outputStat("Largest WCComponent #", largestWCC);
		outputStat("Number of WCComponents", numWCC);
		outputStat("Nodes in largest", WCCnodes[largestWCC]);		
		outputStat("Edges in largest", WCCedges[WCC[largestWCC]]);				
	}
	cerr << "------------------------------------------------------------------" 
	 	 << endl << endl;
} // print

void Graph::load(char * filename, char type) {
	int a, a2, a3, b;
	cerr << "Loading type '" << type << "' from " << filename << "..." << endl;	
	ifstream fin;
	fin.open(filename);
	if(type == 'e') {
		while(fin >> a >> b)
			setEdge(a, b);
	} // if e
	if(type == 'a' || type == 's') {
		fin >> N;
		a = 0;
		a2 = N;
	    while(a2--) {	
			a++;
			fin >> a3;    
			while(a3--) {
				fin >> b;
				if(type == 'a')
			    	setEdge(a, b);	
				if(type == 's')
			    	setUndirEdge(a, b);	
			} // while
		} // while		
	} // if a
	fin.close();
	cerr << "Loading complete." << endl << endl;
	sortEdgeList();
	if(type == 's')
		isUndir = true;
} // load

void Graph::save(char * filename, char type) {
	cerr << "Saving type '" << type << "' to " << filename << "..." << endl;	
	ofstream fout;
	fout.open(filename);
	if(type == 'e') {
		for(int i=1; i<=N; i++)	{
			int z = E[i].size();
			for(int j=0; j<z; j++)
				fout << i << ' ' << E[i][j] << endl;
		} // for
	} // if
	if(type == 'a') {
		fout << N << endl;
		for(int i=1; i<=N; i++)	{
			int z = E[i].size();
			fout << z; 
			for(int j=0; j<z; j++)
				fout << ' ' << E[i][j];
			fout << endl;
		} // for
	} // if	
	if(type == 's') {
		fout << N << endl;	
		for(int i=1; i<=N; i++)	{		
			vector<int> goodlist;
			goodlist.clear();
			int z = (signed)E[i].size();
			for(int j=0; j<z; j++) 
				if(i < E[i][j])
					goodlist.push_back(E[i][j]);
			z = (signed)goodlist.size();
			fout << z;		
			for(int j=0; j<z; j++) 
				fout << ' ' << goodlist[j];
			fout << endl;	
		}
	} // if
	fout.close();
	cerr << "Saving complete." << endl << endl;
} // load

bool Graph::exists(int a) {
	return (OUTDEG[a] > 0 || INDEG[a] > 0);
}

void Graph::makeUndir() {
	if(!isUndir) {
		cerr << "Making undirected..." << endl;		
		vector<int> toseti;
		vector<int> tosetj;

		for(int i=1; i<=N; i++) {
			int z = (signed)E[i].size();
			for(int j=0; j<z; j++) 
				if(!edge(E[i][j], i)) {
					toseti.push_back(E[i][j]);				
					tosetj.push_back(i);
				} // if
		} // for

		int z = (signed)toseti.size();
		for(int i=0; i<z; i++)
			setEdge(toseti[i], tosetj[i]);
			
		sortEdgeList();
	} // if
	cerr << "Graph is undirected." << endl << endl;			
	isUndir = true;	
} // makeUndir

void Graph::calcWCC() { // only works undirected.
	cerr << "Calculating WCC..." << endl;
	int current;
	largestWCC = 0;
	numWCC = 0;
	for(int j=1; j<=N; j++) {
		WCC[j] = -1;
		WCCnodes[j] = 0;
		WCCedges[j] = 0;		
	}
	
	// determine components
	numWCC = 0;
	for(int j=1; j<=N; j++) 
	  if(exists(j)) 
		if(WCC[j] == -1) {
			numWCC++;
			current = j;
			WCC[current] = numWCC;
			WCCnodes[numWCC]++;
			queue<int> qu;
			qu.push(current);
			while(!qu.empty()) {
				current = qu.front();
				qu.pop();
				int z = (signed)E[current].size(); 
				for(int i=0; i<z; i++) {
					if(WCC[E[current][i]] == -1) {
						WCC[E[current][i]] = numWCC;
						WCCnodes[numWCC]++;
						qu.push(E[current][i]);
					}
				} // for i
			} // while
		} // if

	// determine largest
	for(int i=1; i<=N; i++) {
		if(WCCnodes[i] > 0)
			if(WCCnodes[i] > WCCnodes[largestWCC])
				largestWCC = i;
	}

	// determine edges in largest
	for(int j=1; j<=N; j++) 
		if(exists(j))
			WCCedges[WCC[j]] += OUTDEG[j];

	cerr << "WCC calculated." << endl << endl;	
} // WCC
